﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.Windows.Forms;

using WpkgInstaller;

namespace WpkgNotifier
{
    class TrayNotifier : ApplicationContext
    {
        private NotifyIcon notifyIcon;
        private IWpkgMessageRequestHandlerContract channel;
        private EventLog eventLog;

        public TrayNotifier()
        {
            eventLog = new EventLog();
            string eventSourceName = "TrayNotifier";
            string logName = "WpkgNotifier";

            if (!EventLog.SourceExists(eventSourceName))
            {
                EventLog.CreateEventSource(eventSourceName, logName);
            }

            eventLog.Source = eventSourceName;
            eventLog.Log = logName;

            OpenMessageChannel();
                
            MenuItem syncMenuItem = new MenuItem("Sync", new EventHandler(RequestSync));
            MenuItem queryMenuItem = new MenuItem("Query", new EventHandler(RequestQuery));
            MenuItem exitMenuItem = new MenuItem("Exit", new EventHandler(Exit));

            notifyIcon = new NotifyIcon();
            notifyIcon.Text = "WPKG notifier";
            notifyIcon.Icon = WpkgNotifier.Properties.Resources.AppIcon;
            notifyIcon.ContextMenu = new ContextMenu(new MenuItem[]
                { syncMenuItem, queryMenuItem, exitMenuItem });
            notifyIcon.Visible = true;
        }

        private void OpenMessageChannel()
        {
            string address = "net.pipe://localhost/www.ch.cam.ac.uk/WpkgNotifier";

            NetNamedPipeBinding binding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
            EndpointAddress ep = new EndpointAddress(address);
            channel = ChannelFactory<IWpkgMessageRequestHandlerContract>.CreateChannel(binding, ep);
        }

        void RequestSync(object sender, EventArgs e)
        {
            SetToolTip("Syncing");
            WpkgMessageRequestType response = channel.HandleMessage(WpkgMessageRequestType.DO_SYNC);
            SetToolTip("Done");
        }

        void RequestQuery(object sender, EventArgs e)
        {
            SetToolTip("Querying");
            WpkgMessageRequestType response = channel.HandleMessage(WpkgMessageRequestType.RUN_QUERY);
            SetToolTip("Done");
        }

        void Exit(object sender, EventArgs e)
        {
            // We must manually tidy up and remove the icon before we exit.
            // Otherwise it will be left behind until the user mouses over.
            notifyIcon.Visible = false;
            Application.Exit();
        }

        public void SetToolTip(string msg)
        {
            notifyIcon.Text = msg;
        }

    }
}
